package com.ruoyi.ss.callback.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.pay.wx.domain.NotifyEventType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.http.HttpUtils;
import com.ruoyi.ss.callback.domain.CallbackLog;
import com.ruoyi.ss.callback.mapper.CallbackLogMapper;
import com.ruoyi.ss.callback.service.ICallbackService;
import com.ruoyi.ss.changeBalance.service.IChangeBalanceService;
import com.ruoyi.ss.channel.service.IChannelService;
import com.ruoyi.ss.channel.domain.ChannelVO;
import com.ruoyi.ss.dividendDetail.domain.DividendDetail;
import com.ruoyi.ss.dividendDetail.domain.DividendDetailVO;
import com.ruoyi.ss.dividendDetail.service.IDividendDetailService;
import com.ruoyi.ss.order.domain.Order;
import com.ruoyi.ss.order.domain.OrderVO;
import com.ruoyi.ss.orderOper.service.IOrderOperService;
import com.ruoyi.ss.payBill.domain.PayBill;
import com.ruoyi.ss.payBill.domain.PayBillVO;
import com.ruoyi.ss.payBill.domain.enums.PayBillStatus;
import com.ruoyi.ss.payBill.service.PayBillService;
import com.ruoyi.ss.product.domain.ProductVO;
import com.ruoyi.ss.product.service.IProductService;
import com.ruoyi.ss.refund.domain.Refund;
import com.ruoyi.ss.order.service.IOrderService;
import com.ruoyi.ss.refund.service.IRefundService;
import com.ruoyi.ss.user.domain.UserVO;
import com.ruoyi.ss.user.service.IUserService;
import com.wechat.pay.java.core.RSAAutoCertificateConfig;
import com.wechat.pay.java.core.notification.Notification;
import com.wechat.pay.java.core.notification.NotificationConfig;
import com.wechat.pay.java.core.notification.NotificationParser;
import com.wechat.pay.java.core.notification.RequestParam;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.refund.model.RefundNotification;
import com.wechat.pay.java.service.refund.model.Status;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import static com.ruoyi.common.constant.ServiceConstants.*;


/**
 * 订单支付回调 CallbackService 实现类
 */
@Service
public class CallbackServiceImpl implements ICallbackService {

    private static final Logger logger = LoggerFactory.getLogger(CallbackServiceImpl.class);

    @Resource
    private IOrderService orderService;

    @Resource
    private IRefundService etRefundService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Resource
    private CallbackLogMapper callbackLogMapper;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private IChannelService rlChannelService;

    @Autowired
    private IUserService userService;

    @Autowired
    private IChangeBalanceService changeBalanceService;

    @Resource
    private IDividendDetailService dividendDetailService;

    @Autowired
    private IOrderOperService orderOperService;

    @Autowired
    private PayBillService paymentService;

    @Autowired
    private IProductService productService;


    /**
     * 微信支付回调
     */
    @Override
    @Transactional
    public void weChat(HttpServletRequest request) {
        String body = HttpUtils.getBody(request);
        logger.info("【微信支付回调】接收对象（未验签） : {}", JSON.toJSONString(body));
        CallbackLog etCallbackLog = new CallbackLog();
        etCallbackLog.setBody2(body);
        etCallbackLog.setType("1");
        // 解析通知数据
        Notification notification = JSON.parseObject(body, Notification.class);

        //异步保存回调日志
        asynchronousSaveCallbackLog(etCallbackLog);

        // 支付成功通知
        if (NotifyEventType.TRANSACTION_SUCCESS.getValue().equals(notification.getEventType())) {
            // 验签、解密并转换成 Transaction
            Transaction transaction = checkAndParse(request, body, Transaction.class);
            if (Transaction.TradeStateEnum.SUCCESS.equals(transaction.getTradeState())) {
                // 充值成功后的业务处理
                etCallbackLog.setBody(JSON.toJSONString(transaction));
                businessHandle(transaction.getOutTradeNo(), ServiceConstants.PAY_TYPE_WX,DateUtils.toLocalDateTimeByISO(transaction.getSuccessTime()));//业务处理
            }
        }
    }

    /**
     * 支付成功后的业务处理
     *  1. 更改订单状态，订单到期时间修改，账变记录保存至订单
     *  2. 生成配送工单
     *  3. 生成账变记录
     */
    @Override
    public void businessHandle(String outTradeNo, String payType, LocalDateTime payTime) {
        PayBillVO payBillVO = paymentService.selectByPayNo(outTradeNo);
        ServiceUtil.assertion(!PayBillStatus.PAYING.getStatus().equals(payBillVO.getStatus()), "该支付订单不是正在支付的支付订单");

        OrderVO orderVO = orderService.selectRlOrderByOrderNo(payBillVO.getOrderNo());
        ServiceUtil.assertion(orderVO == null, "订单不存在");

        logger.info("【支付回调】订单信息 : {}", JSON.toJSONString(orderVO));

        /**
         *  1. 更新支付订单表，pay_bill
         2. 更新订单表的状态
         3. 更新分账表  分账金额、状态、实际支付金额
         4. 更新用户账变
         5. 订单履历
         */
        Boolean execute = transactionTemplate.execute(e -> {
            updateOrder(orderVO,payTime,payBillVO);
            logger.info("【支付回调】----------更新订单表成功----------");

            updatePayBill(payBillVO,payTime);
            logger.info("【支付回调】----------更新支付订单表成功----------");
            String orderNo = orderVO.getOrderNo();
            if(orderNo.startsWith(ORDER_TYPE_REORDER_PREFIX) ){// 续单 已xd开头的都是续单
                logger.info("【支付回调】----------续费订单----------开始处理业务");

                updateDividendAndChangeBalance(orderVO,payBillVO.getPayNo(),REORDER);
                logger.info("【支付回调-续单】----------更新分账表成功----------");

                // 结束原订单
                closeOriginalOrder(orderVO.getOriginalOrderNo());
            }else if (orderNo.startsWith(ORDER_TYPE_TEA_PREFIX) || orderNo.startsWith(ORDER_TYPE_MAHJONG_PREFIX) || orderNo.startsWith(ORDER_TYPE_BILLIARDS_PREFIX)){// 正常预定订单
                logger.info("【支付回调】----------预约订单----------开始处理业务");

                updateDividendAndChangeBalance(orderVO,payBillVO.getPayNo(),RESERVE);
                logger.info("【支付回调-预订单】----------更新分账表成功----------");

            }else if(orderVO.getOrderNo().startsWith(ServiceConstants.ORDER_TYPE_CONTAINER_PREFIX)){
                logger.info("【支付回调】----------商品订单----------开始处理业务");

                updateDividendAndChangeBalance(orderVO,payBillVO.getPayNo(),PRODUCT_ORDER);

                // 库存减一
                takeOutStock(orderVO);
                logger.info("【支付回调】----------商品订单----------更新库存成功----------");

            } else {
                throw new ServiceException("非法的订单类型");
            }
//            else if (orderVO.getOrderNo().startsWith(ServiceConstants.ORDER_TYPE_RECHARGE_PREFIX)) { // 充值订单
//                logger.info("【支付回调】----------充值订单----------开始处理业务");
//                /**
//                 *   删除用户余额记录，新建一条记录   更新用户余额  余额表
//                 *         产生用户账变
//                 *         产生商户账变
//                 *         更新商户余额和待结算金额
//                 *         充值金额、实际到账金额
//                 *         √订单履历
//                 *         */
//
//                // 用户账变
//                userChangeBalance(orderVO, payBillVO.getPayNo());
//
////                //  todo 商户账变
////                merchantChangeBalance(orderVO, payBillVO.getPayNo());
//
//            }
            createOrderOper(orderVO);
            logger.info("【支付回调】----------创建订单履历成功----------");
            logger.info("=================【微信支付回调】全部结束！！！！！==================");
            return Boolean.TRUE;
        });
        if(Boolean.FALSE.equals(execute))throw new ServiceException("微信支付回调失败");
    }

    private void takeOutStock(OrderVO orderVO) {
        ProductVO product = productService.selectProductByProductId(orderVO.getProductId());
        ServiceUtil.assertion(product == null, "商品不存在");
        int i = productService.updateProductStock(product.getProductId());
        ServiceUtil.assertion(i < 1, "库存不足");
    }

    private void userChangeBalance(OrderVO orderVO, String payNo) {
        UserVO user = userService.selectUserById(orderVO.getUserId());
        ServiceUtil.assertion(user == null, "充值用户不存在");
        /* 记录账变  */
        int i = changeBalanceService.generateChangeBalance(orderVO.getOrderNo(), payNo, ServiceConstants.FLOW_TYPE_INCOME,
                ServiceConstants.RECHARGE, orderVO.getPayFee(), user.getUserId(), user.getUserName(), user.getPhonenumber(),user.getUserType());
        ServiceUtil.assertion(i == 0, "更新记录账变失败");
    }

    private void createOrderOper(OrderVO orderVO) {
        String orderNo = orderVO.getOrderNo();
        BigDecimal payFee = orderVO.getPayFee();
        Long userId = orderVO.getUserId();
        String phone = orderVO.getPhone();
        String status = orderVO.getStatus();
        String desc = getDesc(orderNo, payFee);
        recordOrderHistory(orderNo, status, status, payFee, userId, phone, desc);
    }

    private String getDesc(String orderNo, BigDecimal payFee) {
        String desc;
        // 使用 Map 来简化订单类型和描述的映射
        Map<String, String> orderTypeMap = new HashMap<>();
        orderTypeMap.put(ORDER_TYPE_TEA_REORDER_PREFIX, "茶室订单续单");
        orderTypeMap.put(ORDER_TYPE_MAHJONG_REORDER_PREFIX, "棋牌订单续单");
        orderTypeMap.put(ORDER_TYPE_BILLIARDS_REORDER_PREFIX, "台球订单续单");
        orderTypeMap.put(ORDER_TYPE_TEA_PREFIX, "茶室订单");
        orderTypeMap.put(ORDER_TYPE_MAHJONG_PREFIX, "棋牌订单");
        orderTypeMap.put(ORDER_TYPE_BILLIARDS_PREFIX, "台球订单");
        orderTypeMap.put(ORDER_TYPE_CONTAINER_PREFIX, "商品订单");

        // 遍历 Map 来获取描述
        for (Map.Entry<String, String> entry : orderTypeMap.entrySet()) {
            if (orderNo.startsWith(entry.getKey())) {
                desc = entry.getValue() + "：已支付" + payFee + "元";
                return desc; // 找到匹配后直接返回
            }
        }

        throw new ServiceException("非法的订单类型");
    }

    private void recordOrderHistory(String orderNo, String oldStatus, String newStatus, BigDecimal newFee, Long userId, String phone, String description) {
        if (!orderOperService.recordOrderHistory(orderNo, ServiceConstants.ORDER_OPERATION_PAY, oldStatus, newStatus, BigDecimal.ZERO, newFee, userId, phone, description)) {
            throw new ServiceException("【订单履历-" + description + "】更新订单信息失败，订单号: " + orderNo);
        }
    }


//    private void createOrderOper(OrderVO orderVO) {
//        if(orderVO.getOrderNo().startsWith(ServiceConstants.ORDER_TYPE_RESERVE_PREFIX)){// 预约订单
//            /* 预订单--记录订单履历 */
//            if(!orderOperService.recordOrderHistory(orderVO.getOrderNo(),ServiceConstants.ORDER_OPERATION_PAY,
//                    orderVO.getStatus(),orderVO.getStatus(),BigDecimal.ZERO,orderVO.getPayFee(),orderVO.getUserId(),orderVO.getPhone(),"预约订单支付：已支付 "+orderVO.getPayFee()+"元")){
//                throw new ServiceException("【订单履历-预约订单支付】更新订单信息失败");
//            }
//        } else if (orderVO.getOrderNo().startsWith(ServiceConstants.ORDER_TYPE_REORDER_PREFIX)){// 续费订单
//            if(!orderOperService.recordOrderHistory(orderVO.getOrderNo(),ServiceConstants.ORDER_OPERATION_PAY,
//                    orderVO.getStatus(),orderVO.getStatus(),BigDecimal.ZERO,orderVO.getPayFee(),orderVO.getUserId(),orderVO.getPhone(),"续订单支付：已支付 "+orderVO.getPayFee()+"元")){
//                throw new ServiceException("【订单履历-续订单支付】更新订单信息失败");
//            }
//        } else if (orderVO.getOrderNo().startsWith(ServiceConstants.ORDER_TYPE_RECHARGE_PREFIX)) { // 充值订单
//            if(!orderOperService.recordOrderHistory(orderVO.getOrderNo(),ServiceConstants.ORDER_OPERATION_PAY,
//                    orderVO.getStatus(),orderVO.getStatus(),BigDecimal.ZERO,orderVO.getPayFee(),orderVO.getUserId(),orderVO.getPhone(),"充值订单支付：已支付 "+orderVO.getPayFee()+"元")){
//                throw new ServiceException("【订单履历-充值订单支付】更新订单信息失败");
//            }
//        }else {
//            throw new ServiceException("非法的订单类型");
//        }
//    }

    private void updateDividendAndChangeBalance(OrderVO orderVO,String payNo,String businessType) {
        List<DividendDetailVO> dividendDetailVOS = dividendDetailService.selectRlDividendDetailListByOrderNo(orderVO.getOrderNo());
        ServiceUtil.assertion(dividendDetailVOS.isEmpty(), "分账记录不存在");
        for(DividendDetailVO dividendDetailVO : dividendDetailVOS){
            DividendDetail dividendDetail = new DividendDetail();
            dividendDetail.setStatus(ServiceConstants.DIVIDEND_STATUS_PAID);
            dividendDetail.setDividendAmount(dividendDetailVO.getDividendProportion().multiply(orderVO.getPayFee()).setScale(2, RoundingMode.HALF_UP));
            dividendDetail.setId(dividendDetailVO.getId());

            UserVO user = userService.selectUserById(dividendDetailVO.getPartnerId());
            ServiceUtil.assertion(user == null, "分账用户不存在");

            /* 记录账变  */
            int i = changeBalanceService.generateChangeBalance(orderVO.getOrderNo(), payNo, ServiceConstants.FLOW_TYPE_INCOME,
                    businessType, dividendDetail.getDividendAmount(), user.getUserId(), user.getUserName(), user.getPhonenumber(),user.getUserType());
            ServiceUtil.assertion(i == 0, "更新记录账变失败");

            int i2 = dividendDetailService.updateRlDividendDetail(dividendDetail);
            ServiceUtil.assertion(i2 == 0, "更新分账状态失败");
        }
    }

    private void updatePayBill(PayBillVO payBillVO,LocalDateTime payTime) {
        PayBill updatedPayBill =  new PayBill();
        updatedPayBill.setStatus(PayBillStatus.PAY_SUCCESS.getStatus());
        updatedPayBill.setPayTime(payTime);
        updatedPayBill.setPayId(payBillVO.getPayId());
        int i = paymentService.updatePayBill(updatedPayBill);
        ServiceUtil.assertion(i == 0, "更新支付订单状态失败");
    }

    private void updateOrder(OrderVO orderVO,LocalDateTime payTime,PayBillVO payBill) {
        Order updateOrder = new Order();
        updateOrder.setOrderId(orderVO.getOrderId());
        updateOrder.setPaid(ServiceConstants.ORDER_PAY_STATUS_PAID);
        // todo 如果支付时间在预约开始时间之前，则状态是待使用，如果在预约开始时间之后，则状态是使用中，如果支付时间在预约结束时间之后。这种请在下单前就做做判断，避免出现这种情况
        LocalDateTime reserveStartTime = DateUtils.toLocalDateTime(orderVO.getReserveStartTime());
        String status;
        if(payTime.isBefore(reserveStartTime)){
            status = ServiceConstants.ORDER_STATUS_PAYMENT_SUCCESS;
        }else{
            status = ServiceConstants.ORDER_STATUS_IN_USE;
        }
        updateOrder.setStatus(status);
        updateOrder.setPayTime(payTime);
        updateOrder.setPayChannel(payBill.getChannelId());
        ChannelVO channelVO = rlChannelService.selectChannelByChannelId(payBill.getChannelId());
        updateOrder.setChannelName(channelVO.getName());
        updateOrder.setChannelCost(payBill.getChannelCost());
        updateOrder.setPayId(payBill.getPayId());
        int i = orderService.updateRlOrder(updateOrder);
        ServiceUtil.assertion(i == 0, "更新订单状态失败");
    }


    /** 结算原订单 */
    private void closeOriginalOrder(String originalOrderNo) {
            Order updateOriginalOrder = new Order();
            updateOriginalOrder.setOrderNo(originalOrderNo);
            updateOriginalOrder.setStatus(ServiceConstants.ORDER_STATUS_COMPLETED);
            int i = orderService.updateRlOrderByOrderNo(updateOriginalOrder);
            ServiceUtil.assertion(i==0,"更新原订单状态失败");
    }


    private BigDecimal getCost(ChannelVO channel,BigDecimal payFee) {
        // 获取到渠道成本
        BigDecimal costRate = channel.getCostRate();
        BigDecimal bigDecimal = costRate.divide(new BigDecimal(100), 6, BigDecimal.ROUND_HALF_UP);
        BigDecimal cost = bigDecimal.multiply(payFee).setScale(2, BigDecimal.ROUND_HALF_UP);
        logger.info("【保存资金流水记录--订单支付】 成本==============bigDecimal=====================："+bigDecimal);
        logger.info("【保存资金流水记录--订单支付】 成本==============cost=====================："+cost);
        return cost;
    }



    private void asynchronousSaveCallbackLog(CallbackLog etCallbackLog) {
        //开异步线程保存回调参数
        scheduledExecutorService.schedule(() -> {
            logger.info("【微信支付回调】异步保存回调参数");
            etCallbackLog.setCreateTime(DateUtils.getNowDate());
            int i = callbackLogMapper.insertEtCallbackLog(etCallbackLog);
            if(i>0){
                logger.info("【微信支付回调】异步保存回调参数成功");
            }
        }, 0, TimeUnit.SECONDS);
    }


    /**
     * 微信退款回调
     */
    @Override
    @Transactional
    public void weChatRefund(HttpServletRequest request) {
        String body = HttpUtils.getBody(request);
        CallbackLog etCallbackLog = new CallbackLog();
        etCallbackLog.setBody2(body);
        etCallbackLog.setType("2");
        logger.info("【微信退款回调】接收对象 : {}", JSON.toJSONString(body));
        // 解析通知数据
        Notification notification = JSON.parseObject(body, Notification.class);
        logger.info("【微信退款回调】转换成notification: {}", JSON.toJSONString(notification));
        // 退款成功通知
        if (NotifyEventType.REFUND_SUCCESS.getValue().equals(notification.getEventType())) {
            // 验签、解密并转换成 RefundNotification
            RefundNotification refundNotification = checkAndParse(request, body, RefundNotification.class);
            logger.info("【微信退款回调】转换成RefundNotification: {}", JSON.toJSONString(refundNotification));
            etCallbackLog.setBody(JSON.toJSONString(refundNotification));
            if (Status.SUCCESS.equals(refundNotification.getRefundStatus())) {
                // 退款成功后  改变 etRefundService
                // 如果outRefundNo没有值，说明是押金退款，直接新增退款记录，  还是在还车时新建退款记录？
                String outRefundNo = refundNotification.getOutRefundNo();
                Refund rlRefund = new Refund();
                rlRefund.setRefundNo(outRefundNo);
                rlRefund.setRefundResult(Constants.SUCCESS2);
                int i = etRefundService.updateEtRefundByRefundNo(rlRefund);
                if(i==0){
                    logger.error("【微信退款回调】更新退款单失败");
                    throw new ServiceException("【微信退款回调】更新退款单失败");
                }
                logger.info("【微信退款回调】全部结束！！！！！");
            }
        }
        //异步保存回调日志
        asynchronousSaveCallbackLog(etCallbackLog);
    }

    /**
     * 验签并解析
     * @param request 请求
     * @param body 请求体
     * @param clazz 返回值类型
     */
    private <T> T checkAndParse(HttpServletRequest request, String body, Class<T> clazz) {
        // 构造 RequestParam
        RequestParam requestParam = new RequestParam.Builder()
                .serialNumber(request.getHeader("Wechatpay-Serial"))
                .nonce(request.getHeader("Wechatpay-Nonce"))
                .signature(request.getHeader("Wechatpay-Signature"))
                .timestamp(request.getHeader("Wechatpay-Timestamp"))
                .body(body)
                .build();
        // 验签
        String wechatpaySerial = request.getHeader("Wechatpay-Serial");
        NotificationParser notificationParser = getNotificationParser(wechatpaySerial);
        return notificationParser.parse(requestParam, clazz);
    }


    public NotificationParser getNotificationParser(String wechatpaySerial) {
        // 根据证书序列号获取运营商
        ChannelVO channelVO = rlChannelService.selectChannelBySerialNumber(wechatpaySerial);
        if(ObjectUtil.isNull(channelVO)){
            throw new ServiceException("【微信支付回调】根据证书序列号获取支付渠道失败");
        }else{
            NotificationConfig config = new RSAAutoCertificateConfig.Builder()
                    .merchantId(channelVO.getMerchantId())
                    .privateKeyFromPath(channelVO.getPrivateKeyPath())
                    .merchantSerialNumber(channelVO.getMerchantSerialNumber())
                    .apiV3Key(channelVO.getApiV3Key())
                    .build();
            return new NotificationParser(config);
        }
    }
}
