package com.ruoyi.ss.order.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.BaseValidator;
import com.ruoyi.common.core.domain.ValidateResult;
import com.ruoyi.common.core.domain.entity.User;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.order.domain.OrderBO;
import com.ruoyi.ss.order.domain.OrderQuery;
import com.ruoyi.ss.order.domain.OrderVO;
import com.ruoyi.ss.order.mapper.OrderMapper;
import com.ruoyi.ss.order.service.IOrderService;
import com.ruoyi.ss.order.service.IOrderValidator;
import com.ruoyi.ss.room.domain.RoomVO;
import com.ruoyi.ss.room.service.IRoomService;
import com.ruoyi.ss.store.domain.StoreVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Objects;

@Service
public class IOrderValidatorImpl extends BaseValidator implements IOrderValidator {

    @Autowired
    private IOrderService orderService;

    @Resource
    private OrderMapper orderMapper;

    @Autowired
    private IOrderValidator orderValidator;

    @Autowired
    private IRoomService roomService;

    @Override
    public ValidateResult preAddOrder(OrderBO bo) {
        if (bo == null) {
            return error("数据不能为空");
        }

        // 营业时间判断
        StoreVO store = bo.getStore();
        Date businessTimeStart = store.getBusinessTimeStart();
        Date businessTimeEnd = store.getBusinessTimeEnd();
        if (businessTimeStart == null || businessTimeEnd == null) {
            return error("营业时间为空");
        }

        // 房间还有进行中的订单
//        RoomVO room = bo.getRoom();
//        boolean inProgressOrder = getInProgressOrder(room.getRoomId());
//        if (inProgressOrder) {
//            return error("该房间还有进行中的订单");
//        }
//
//        // 该房间在预约时段是否在订单
//        if(bo.getType().equals(ServiceConstants.ORDER_TYPE_RESERVE)){
//            if(isReserved(room.getRoomId(), bo.getOrder().getReserveStartTime(), bo.getOrder().getReserveEndTime()) > 0){
//                return error("该房间在预约时段存在订单");
//            }
//            // 价格已发生变化
//            if(bo.getFeeRule().getPrice().compareTo(bo.getOrder().getPrice()) != 0){
//                return error("价格已发生变化");
//            }
//            // 价格未配置
//            if(ObjectUtil.isNull(bo.getFeeRule().getPrice()) || bo.getFeeRule().getPrice().compareTo(BigDecimal.ZERO) == 0){
//                return error("价格未配置");
//            }
//            // 小时未配置
//            if(ObjectUtil.isNull(bo.getFeeRule().getHours()) || bo.getFeeRule().getHours() <= 0){
//                return error("小时未配置");
//            }
//        }else{
//            if(isReserved(room.getRoomId(), bo.getOriginalOrder().getReserveStartTime(), bo.getOriginalOrder().getReserveEndTime()) > 0){
//                return error("该房间在预约时段存在订单");
//            }
//        }
        return success();
    }



    @Override
    public void validateChangeRoom(OrderQuery orderQuery) {
        if(StrUtil.isBlank(orderQuery.getOrderNo())){
            throw new ServiceException("订单号不能为空");
        }
        if(orderQuery.getNewRoomId() == null){
            throw new ServiceException("新包间id不能为空");
        }
        OrderVO orderVO = orderService.selectRlOrderByOrderNo(orderQuery.getOrderNo());
        ServiceUtil.assertion(orderVO == null, "订单不存在");
        ServiceUtil.assertion(orderVO.getPaid() == null || orderVO.getPaid().equals(ServiceConstants.ORDER_PAY_STATUS_NON_PAYMENT), "订单未支付");
        ServiceUtil.assertion(orderVO.getStatus() == null || !orderVO.getStatus().equals(ServiceConstants.ORDER_STATUS_PAYMENT_SUCCESS), "订单使用状态");

        RoomVO newRoom = roomService.selectERoomByRoomId(orderQuery.getNewRoomId());
        ServiceUtil.assertion(newRoom == null, "新包间不存在");
        ServiceUtil.assertion(newRoom.getStoreId() == null || !newRoom.getStoreId().equals(orderVO.getStoreId()), "新包间不属于当前店铺，不能更换");
        ServiceUtil.assertion(newRoom.getStatus() == null || !newRoom.getStatus().equals(ServiceConstants.ROOM_STATUS_FREE), "新包间非空闲状态，不能更换");
        ServiceUtil.assertion(newRoom.getType() == null || !newRoom.getType().equals(orderVO.getRoomType()), "新包间类型与原包间类型不一致，不能更换");
        ServiceUtil.assertion(Objects.equals(orderVO.getRoomId(), newRoom.getRoomId()), "新包间与原包间相同，无需更换");

        // 新包间在订单时间内是否有预定单  在指定时间段内是否有订单
        Date reserveStartTime = orderVO.getReserveStartTime();
        Date reserveEndTime = orderVO.getReserveEndTime();// 是否被预定  已
        int isReserved = orderValidator.isReserved(newRoom.getRoomId(), reserveStartTime, reserveEndTime);
        String reserveStartTimeString =  new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(reserveStartTime);
        String reserveEndTimeString =  new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(reserveEndTime);
        ServiceUtil.assertion(isReserved > 1, "新包间在【"+reserveStartTimeString+"】~【"+reserveEndTimeString+"】时间段内有预定单，不能更换");

    }

    @Override
    public void checkDividendProportions(List<User> dividendUsers) {
        // 使用Stream处理累加所有用户的分账比例，并且检查是否大于1
        BigDecimal totalProportion = dividendUsers.stream()
                // 映射成 BigDecimal 的分账比例
                .map(User::getDividendProportion)
                // 累加所有比例
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        // 校验
        ServiceUtil.assertion(totalProportion.compareTo(BigDecimal.ONE) > 0,"分账总比例不能大于100%");
    }

    /**
     * 进行中的订单校验
     */
    @Override
    public boolean getInProgressOrder(Long roomId) {
        int inProgressOrder = orderMapper.getInProgressOrder(roomId);
        return inProgressOrder > 0;
    }


    /**
     * 进行中的订单校验
     */
    @Override
    public boolean getInProgressOrderByIds(List<Long> roomIds) {
        if (roomIds.isEmpty()) {
            return false;
        }
        int inProgressOrder = orderMapper.getInProgressOrderByIds(roomIds);
        return inProgressOrder > 0;
    }

    /**
     * 时间段内是否有预定单
     */
    @Override
    public int isReserved(Long roomId, Date startTime, Date endTime) {
        return orderMapper.isReserved(roomId, startTime, endTime);
    }


}
