package com.ruoyi.ss.store.service.impl;

import com.ruoyi.common.utils.map.GeoUtils;
import com.ruoyi.common.utils.CommonUtil;
import com.ruoyi.ss.device.domain.DeviceQuery;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.order.domain.OrderQuery;
import com.ruoyi.ss.order.service.IOrderService;
import com.ruoyi.ss.room.domain.RoomQuery;
import com.ruoyi.ss.room.service.IRoomService;
import com.ruoyi.ss.store.domain.StoreVO;
import com.ruoyi.ss.store.service.IStoreAssembler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Comparator;
import java.util.List;

import static com.ruoyi.common.constant.ServiceConstants.*;

/**
 * @author qzz
 * 2024/4/29
 */
@Service
public class StoreAssemblerImpl implements IStoreAssembler {

    @Autowired
    private DeviceService deviceService;

    @Autowired
    private IRoomService roomService;

    @Autowired
    private IOrderService orderService;

    /**
     * 拼接设备数量
     *
     * @param list 店铺列表
     */
    @Override
    public void assembleRuleInfo(List<StoreVO> list) {
//        if (CollectionUtils.isEmptyElement(list)) {
//            return;
//        }
//        Map<Long, Integer> countMap = deviceService.selectCountMapByStoreIds(list.stream().map(StoreVo::getStoreId).collect(Collectors.toList()));
    }

    /**
     * 拼接距离
     * @param list 店铺列表
     * @param lon 经度
     * @param lat 纬度
     */
    @Override
    public void assembleDistance(List<StoreVO> list, String lon, String lat) {
        if (list.isEmpty()) {
            return;
        }
        for (StoreVO eStoreVO : list) {
            eStoreVO.setDistance(CommonUtil.getDistance(lon, lat, eStoreVO.getLng(), eStoreVO.getLat()));
        }
        // 按距离升序排序
        list.sort(Comparator.comparingDouble(StoreVO::getDistance));
    }

    /**
     * 拼接统计数据
     * @param vo 店铺对象
     *
     */
    @Override
    public void assembleStatisticalData(StoreVO vo) {
        if (vo == null) {
            return;
        }

        // 拼接今日营收、本月营收、总营收
        setIncome(vo);

        // 房间总数、正在使用中的房间数、设施总数、正在使用中的设施数
        roomNum(vo);

        // 统计设备数量，设备表中的store_id，投放状态
        deviceNum(vo);

    }

    private void setIncome(StoreVO vo) {
        OrderQuery orderQuery = new OrderQuery();
        orderQuery.setStoreId(vo.getStoreId());
        BigDecimal totalRevenue = orderService.getOrderFee(orderQuery);
        vo.setTotalIncome(totalRevenue);// 总营收
        vo.setTodayIncome(orderService.getRevenueByTimeType(TIME_LIMIT_TODAY,orderQuery));
        vo.setMonthIncome(orderService.getRevenueByTimeType(TIME_LIMIT_MONTH,orderQuery));
    }

    private void roomNum(StoreVO vo) {
        RoomQuery query = new RoomQuery();
        query.setStoreId(vo.getStoreId());
        query.setType2(ROOM_TYPE2_PRIVATE_ROOM);
        vo.setTotalRoomNum(roomService.getCount(query));
        query.setStatus(ROOM_STATUS_INUSE);
        vo.setUsingRoomNum(roomService.getCount(query));
        query.setType2(ROOM_TYPE2_LOBBY_DEVICE);
        vo.setUsingFacilityNum(roomService.getCount(query));
        query.setStatus(null);
        vo.setTotalFacilities(roomService.getCount(query));
    }

    private void deviceNum(StoreVO vo) {
        DeviceQuery query1 = new DeviceQuery();
        query1.setStoreId(vo.getStoreId());
        int i = deviceService.selectCount(query1);
        vo.setTotalDeviceNum(i);
    }


}
